package main

import (
  "log"
  "os"
  "fmt"
  "github.com/gin-gonic/gin"
  MQTT "github.com/eclipse/paho.mqtt.golang"
  "github.com/google/uuid"
  "net/http"
  "locsrv/database"
)

/*
{
  "trigger":"enter",
  "device":"aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
  "device_model":"iPhone13,1",
  "timestamp":"1705162497.0",
  "latitude":"51.0",
  "id":"test",
  "longitude":"7.0",
  "device_type":"iPhone"
}
*/

type locativeEvent struct {
  Trigger string `json:"trigger"`
  Device string `json:"device"`
  Location string `json:"id"`
  Latitude string `json:"latitude"`
  Longitude string `json:"longitude"`
  Person string
}


func main() {
  router := gin.Default()

  ch := make(chan locativeEvent)
  dbh := database.New()

  mqttOpts := MQTT.NewClientOptions().
    AddBroker(os.Getenv("MQTT_BROKER")).
    SetClientID(fmt.Sprintf("locsrv-%s", uuid.New())).
    SetConnectRetry(true)
  mqttClient := MQTT.NewClient(mqttOpts)
  if token := mqttClient.Connect(); token.Wait() && token.Error() != nil {
    log.Fatalf("Unable to connect to broker, error %v", token.Error())
  }
  mqttTopic := os.Getenv("MQTT_TOPIC")
  if mqttTopic != "" {
    log.Fatalf("No topic set")
  }

  go func() {
    for {
      select {
      case event := <- ch:
        person, err := dbh.GetPersonById(event.Device)
        event.Person = person
        if err != nil {
          log.Printf("Person unknown: %v", err)
        } 
        log.Printf("Trigger: %s, Device: %s, Location: %s, Person: %s, Latitude: %s, Longitude: %s", event.Trigger, event.Device, event.Location, event.Person, event.Latitude, event.Longitude)
      }
    }
  }()

  router.POST("/location", func(c *gin.Context) {
    var body locativeEvent

    err := c.ShouldBindJSON(&body)
    if err != nil {
      c.JSON(http.StatusBadRequest, gin.H { "error": err.Error() })
      return
    }

    var message string
    select {
    case ch <- body:
      message = "message forwarded"
    default:
      log.Println("Channel full, message lost")
      message = "message lost"
    }

    c.JSON(http.StatusOK, gin.H { "message": message })
  })

  router.Run(":8080")
}

